---
title: 平台选项设置
permalink: /docs/zh/host-option.html
key: docs-host-option
---

在 fijkplayer 中，特意为播放器的属性设置接口增加了一个类别，`FijkOption.hostCategory`。  
用于设置平台相关的一些内容。本节文档给出所有的 hostCategory option。


## 音频 Audio Focus

**只用于 Android，其他平台无作用**

```dart
await player.setOption(FijkOption.hostCategory, "request-audio-focus", 1);
```
默认值是 0，不产生作用。

如果设置为 1，会在播放器进入 `started` 状态后调用系统 requestAudioFocus 接口。  
`streamType` 参数使用 `AudioManager.STREAM_MUSIC`，durationHint 参数使用 `AudioManager.AUDIOFOCUS_GAIN`。

```dart
await player.setOption(FijkOption.hostCategory, "release-audio-focus", 1);
```
默认值是 0，不产生作用。

如果设置为 1，会在播放器离开 `started` 状态后调用系统 abandonAudioFocus 接口。 

也可以使用接口 
[FijkPlugin.requestAudioFocus](https://pub.dev/documentation/fijkplayer/latest/fijkplayer/FijkPlugin/requestAudioFocus.html) 和 
[FijkPlugin.releaseAudioFocus](https://pub.dev/documentation/fijkplayer/latest/fijkplayer/FijkPlugin/releaseAudioFocus.html) 
管理 Android Audio Focus。

但注意不要同时使用 host option 和 FijkPlugin 中的接口

## 屏幕常量

**用于 Android 和 iOS 平台**

```dart
await player.setOption(FijkOption.hostCategory, "request-screen-on", 1);
```
默认值是 0，不产生作用。

如果设置为 1， 会在播放器进入 `started` 状态后调用系统接口保持屏幕常亮，
当播放器离开 `started` 状态时，会调用系统接口关闭屏幕常量。

也可以通过 [FijkPlugin.keepScreenOn](https://pub.dev/documentation/fijkplayer/latest/fijkplayer/FijkPlugin/keepScreenOn.html) 手动管理屏幕常量，但不要同时使用这两种方法，容易混乱。


## 截图

**用于 Android 和 iOS 平台**
```dart
await player.setOption(FijkOption.hostCategory, "enable-snapshot", 1);
```
如果不进行此设置，FijkPlayer 的截图接口会没有作用。 option 要在开始播放前进行设置。
开启截图功能后，有额外的性能消耗。所以仅在需要的时候设置播放器开启结果。

